#include <iostream>
#include <fstream>
#include "gluvi.h"
#include "simplelexer.h"

using namespace std;

struct Segment{
   float x0, y0, x1, y1;
};

struct Frame
{
   vector<Segment> segment;
};


void read_animation(const char *animfile, vector<Frame> &anim)
{
   ifstream in(animfile);
   SimpleLexer lex(in);
   SimpleToken tok;

   int nframes;
   lex.read(tok);
   if(tok.type!=TOKEN_INT) { cerr<<"Syntax of animation output file - expecting integer for number of frames"<<endl; exit(1); }
   nframes=tok.data.int_value;
   anim.resize(nframes);

   for(int i=0; i<nframes; ++i){
      int nlinks;
      lex.read(tok);
      if(tok.type!=TOKEN_INT) { cerr<<"Syntax of animation output file - expecting integer for number of links in frame "<<i<<endl; exit(1); }
      nlinks=tok.data.int_value;
      anim[i].segment.resize(nlinks);
      
      for(int j=0; j<nlinks; ++j){
	 int link;
         lex.read(tok);
         if(tok.type!=TOKEN_INT) { cerr<<"Syntax of animation output file - expecting integer for link number in frame "<<i<<endl; exit(1); }
	 link=tok.data.int_value;
	 if(link<0 || link>=nlinks) { cerr<<"Bad link number "<<link<<" in frame <<"<<i<<endl; exit(1); }

	 lex.read(tok);
	 if(tok.type==TOKEN_INT) anim[i].segment[link].x0=(float)tok.data.int_value;
	 else if(tok.type==TOKEN_FLOAT) anim[i].segment[link].x0=tok.data.float_value;
	 else { cerr<<"Syntax of animation output file - expecting number for x0 in link "<<link<<" of frame "<<i<<endl; exit(1); }

	 lex.read(tok);
	 if(tok.type==TOKEN_INT) anim[i].segment[link].y0=(float)tok.data.int_value;
	 else if(tok.type==TOKEN_FLOAT) anim[i].segment[link].y0=tok.data.float_value;
	 else { cerr<<"Syntax of animation output file - expecting number for y0 in link "<<link<<" of frame "<<i<<endl; exit(1); }

	 lex.read(tok);
	 if(tok.type==TOKEN_INT) anim[i].segment[link].x1=(float)tok.data.int_value;
	 else if(tok.type==TOKEN_FLOAT) anim[i].segment[link].x1=tok.data.float_value;
	 else { cerr<<"Syntax of animation output file - expecting number for x1 in link "<<link<<" of frame "<<i<<endl; exit(1); }

	 lex.read(tok);
	 if(tok.type==TOKEN_INT) anim[i].segment[link].y1=(float)tok.data.int_value;
	 else if(tok.type==TOKEN_FLOAT) anim[i].segment[link].y1=tok.data.float_value;
	 else { cerr<<"Syntax of animation output file - expecting number for y1 in link "<<link<<" of frame "<<i<<endl; exit(1); }
      }
   }
}

vector<Frame> animation;
unsigned int frame=0;

void display(void)
{
   glColor3f(1,1,1);
   glBegin(GL_LINES);
   for(unsigned int seg=0; seg<animation[frame].segment.size(); ++seg){
      glVertex2f(animation[frame].segment[seg].x0, animation[frame].segment[seg].y0);
      glVertex2f(animation[frame].segment[seg].x1, animation[frame].segment[seg].y1);
   }
   glEnd();
}

struct CameraButton : public Gluvi::Button {
   CameraButton(const char *t) : Gluvi::Button(t) {}
   void action()
   {
      ofstream camera_output("stick_figure_camera.rib");
      Gluvi::camera->export_rib(camera_output);
   }
};

void special_key_handler(int key, int x, int y)
{
   switch(key){
      case GLUT_KEY_LEFT:
	 if(frame>0){
	    --frame;
	    glutPostRedisplay();
	 }
         break;
      case GLUT_KEY_RIGHT:
	 if(frame+1<animation.size()){
	    ++frame;
	    glutPostRedisplay();
	 }
	 break;
      default:
	 ;
   }
}

int main(int argc, char **argv)
{
   Gluvi::init("stick_figure_opengl", &argc, argv);

   if(argc!=2){
      cerr<<"Expecting one argument: animation_file"<<endl;
      return 1;
   }

   read_animation(argv[1], animation);

   glutSpecialFunc(special_key_handler);
   Gluvi::PanZoom2D cam;
   Gluvi::camera=&cam;
   Gluvi::userDisplayFunc=display;
   Gluvi::StaticText keytext("use left/right arrows for frame control");
   Gluvi::root.list.push_back(&keytext);
   CameraButton cambutton("Export camera to RIB");
   Gluvi::root.list.push_back(&cambutton);
   Gluvi::run();

   return 0;
}

