#ifndef SKELETON2D_H
#define SKELETON2D_H

#include <iostream>
#include <vector>

struct Link
{
   float length;                  // how long the link is
   int nchild;                    // number of child attachments (don't count the parent)
   std::vector<int> child;        // indices of child links
   std::vector<float> fraction;   // fraction of the way along this link each child attachment is made

   Link()
      : length(0), nchild(0)
   {}

   void add_attachment(int newchild, float newfraction)
   {
      child.push_back(newchild);
      fraction.push_back(newfraction);
      ++nchild;
   }
};

struct ControlPoint
{
   float t;       // time of control point - the knot value
   float v;       // value
   float dvminus; // slope for earlier time than t
   float dvplus;  // slope for later time than t

   ControlPoint(float t_=0, float v_=0, float dvminus_=0, float dvplus_=0)
      : t(t_), v(v_), dvminus(0), dvplus(0)
   {}

   void set(float t_, float v_, float dvminus_=0, float dvplus_=0)
   {
      t=t_;
      v=v_;
      dvminus=dvminus_;
      dvplus=dvplus_;
   }

   bool operator<(const ControlPoint &cp) const
   { return t<cp.t; }
};

struct HermiteSpline
{
   std::vector<ControlPoint> control;

   void sort();
   void compute_Catmull_Rom_slopes();
   float operator()(float t);               // evaluation of the spline at time t
};

struct Skeleton2D
{
   std::vector<Link> link;                  // skeleton definition
   int nframes;                             // number of frames in animation
   HermiteSpline link0x, link0y;            // keyframe position data
   std::vector<HermiteSpline> joint;        // keyframe angle data for each link

   Skeleton2D()
      : nframes(0)
   {}

   void read_skeleton(std::istream &input);
   void read_keyframes(std::istream &input);
   void compute_Catmull_Rom_slopes();
   void write_animation(std::ostream &output);
};

#endif
