#include "simplelexer.h"
#include <cstdio>

using namespace std;

void SimpleToken::
set(TokenType type_)
{
   clear();
   type=type_;
   data.identifier_string=0;
   data.float_value=0;
   data.int_value=0;
}

void SimpleToken::
set(TokenType type_, vector<char> &string)
{
   clear();
   type=type_;
   data.identifier_string=new char[string.size()+1];
   unsigned int i;
   for(i=0; i<string.size(); ++i)
      data.identifier_string[i]=string[i];
   data.identifier_string[i]=0; // null-terminated
}

void SimpleToken::
set(TokenType type_, double value)
{
   clear();
   type=type_;
   data.float_value=value;
}

void SimpleToken::
set(TokenType type_, int value)
{
   clear();
   type=type_;
   data.int_value=value;
}

void SimpleToken::
clear()
{
   if(type==TOKEN_ERROR || type==TOKEN_IDENTIFIER){
      delete[] data.identifier_string;
      data.identifier_string=0;
   }
}

static int int_of_charvector(const vector<char> &s)
{
   int v;
   char *str=new char[s.size()+1];
   unsigned int i;
   for(i=0; i<s.size(); ++i)
      str[i]=s[i];
   str[i]=0;
   sscanf(str, "%d", &v);
   delete[] str;
   return v;
}

static double double_of_charvector(const vector<char> &s)
{
   double v;
   char *str=new char[s.size()+1];
   unsigned int i;
   for(i=0; i<s.size(); ++i)
      str[i]=s[i];
   str[i]=0;
   sscanf(str, "%lf", &v);
   delete[] str;
   return v;
}

void SimpleLexer::
read(SimpleToken &tok)
{
   char c;
   enum {MODE_UNKNOWN, MODE_ERROR, MODE_MINUS, MODE_NUMBER, MODE_MANTISSA, MODE_E,
         MODE_EMINUS, MODE_EXPONENT, MODE_IDENTIFIER} mode=MODE_UNKNOWN;
   vector<char> token_string;

   for(;;){
      input.get(c);
      if(!input.good()){ // on end of input ////////////////////////////////////////////
         switch(mode){
            case MODE_UNKNOWN:
               tok.set(TOKEN_EOF); return;
            case MODE_ERROR: case MODE_MINUS: case MODE_E: case MODE_EMINUS: // return error string
               tok.set(TOKEN_ERROR, token_string); return;
            case MODE_NUMBER: // return int
               tok.set(TOKEN_INT, int_of_charvector(token_string)); return;
            case MODE_MANTISSA: case MODE_EXPONENT: // return float
               tok.set(TOKEN_FLOAT, double_of_charvector(token_string)); return;
            case MODE_IDENTIFIER: // return identifier string
               tok.set(TOKEN_IDENTIFIER, token_string); return;
         }
      }else switch(c){ // on a good character ///////////////////////////////////////

         case '-':
            token_string.push_back(c);
            switch(mode){
               case MODE_UNKNOWN:
                  mode=MODE_MINUS;
                  break;
               case MODE_IDENTIFIER:
                  break;
               case MODE_E:
                  mode=MODE_EMINUS;
                  break;
               default:
                  mode=MODE_ERROR;
                  break;
            }
            break;

         case '0': case '1': case '2': case '3': case '4': case '5': case '6': case '7': case '8': case '9':
            token_string.push_back(c);
            switch(mode){
               case MODE_UNKNOWN: case MODE_MINUS:
                  mode=MODE_NUMBER;
                  break;
               case MODE_E: case MODE_EMINUS:
                  mode=MODE_EXPONENT;
                  break;
               default:
                  break;
            }
            break;

         case '.':
            token_string.push_back(c);
            switch(mode){
               case MODE_UNKNOWN: case MODE_NUMBER: case MODE_MINUS:
                  mode=MODE_MANTISSA;
                  break;
               case MODE_IDENTIFIER:
                  break;
               default:
                  mode=MODE_ERROR;
            }
            break;

         case 'e': case 'E':
            token_string.push_back(c);
            switch(mode){
               case MODE_NUMBER: case MODE_MANTISSA:
                  mode=MODE_E;
                  break;
               case MODE_IDENTIFIER:
                  break;
               case MODE_UNKNOWN:
                  mode=MODE_IDENTIFIER;
                  break;
               default:
                  mode=MODE_ERROR;
            }
            break;

         case ' ': case '\t': case '\r': case '\n':
            switch(mode){
               case MODE_UNKNOWN:
                  break; // keep going
               case MODE_ERROR: case MODE_MINUS: case MODE_E: case MODE_EMINUS: // return error string
                  tok.set(TOKEN_ERROR, token_string); return;
               case MODE_NUMBER: // return int
                  tok.set(TOKEN_INT, int_of_charvector(token_string)); return;
               case MODE_MANTISSA: case MODE_EXPONENT: // return float
                  tok.set(TOKEN_FLOAT, double_of_charvector(token_string)); return;
               case MODE_IDENTIFIER: // return identifier string
                  tok.set(TOKEN_IDENTIFIER, token_string); return;
            }
            break;

         case '#': // ignore the rest of the line as a comment
            do input.get(c);
            while(input.good() && c!='\n' && c!='\r');
            break;

         default:
            token_string.push_back(c);
            switch(mode){
               case MODE_UNKNOWN:
                  mode=MODE_IDENTIFIER;
                  break;
               case MODE_IDENTIFIER:
                  break;
               default:
                  mode=MODE_ERROR;
            }
            break;
      }
   }
}

