#ifndef SIMPLELEXER_H
#define SIMPLELEXER_H

#include <iostream>
#include <vector>

enum TokenType {TOKEN_EOF, TOKEN_ERROR, TOKEN_IDENTIFIER, TOKEN_FLOAT, TOKEN_INT};

struct SimpleToken
{
   TokenType type;
   union {
      char *identifier_string; // note: it's the responsibility of the user to delete[] this storage when finished with it
      double float_value;
      int int_value;
   } data;

   SimpleToken() : type(TOKEN_INT) {}
   ~SimpleToken() { clear(); }
   void set(TokenType type_);
   void set(TokenType type_, std::vector<char> &string);
   void set(TokenType type_, double value);
   void set(TokenType type_, int value);
   void clear(); // call to free up string storage after finished with a TOKEN_ERROR or TOKEN_IDENTIFIER

private:
	// to avoid inadverdent memory leaks - don't use these member functions
	SimpleToken(const SimpleToken &copy) {}
	SimpleToken &operator=(const SimpleToken &copy) {return *this;}
};

struct SimpleLexer
{
   std::istream &input;

   SimpleLexer(std::istream &input_)
      : input(input_)
   {}

   void read(SimpleToken &tok);
};

#endif
